------------------ Player Class --------------------

Player = {}
function Player:new()
   local o = {}
   o.state = {}
   o.state.isOnTheFloor = false
   o.walkSpeed = 250
   o.jumpImpulse = 1000
   self.__index = self
   setmetatable(o, self)
   return o
end

function Player:beginCollide(b, coll)
   print("   self.state.isOnTheFloor = true")
   self.state.isOnTheFloor = true
end

function Player:endCollide(b, coll)
   print("   self.state.isOnTheFloor = false")
   self.state.isOnTheFloor = false
end

function Player:jump()
   if self.state.isOnTheFloor then
      local x, _ = self.body:getLinearVelocity()
      self.body:setLinearVelocity(x, 0)
      self.body:applyLinearImpulse(0, -self.jumpImpulse)
   end
end

function Player:walkRight()
   local _, y = self.body:getLinearVelocity()
   self.body:setLinearVelocity(self.walkSpeed, y)
end

function Player:walkLeft()
   local _, y = self.body:getLinearVelocity()
   self.body:setLinearVelocity(-self.walkSpeed, y)
end

function Player:stop()
   local _, y = self.body:getLinearVelocity()
   self.body:setLinearVelocity(0, y)
end

------------------ Categories -------------------
Catg = { _indx_ = 0 }

function Catg:next()
   Catg._indx_ = Catg._indx_ + 1
   return Catg._indx_
end

function Catg:reset()
   Catg._indx_ = 0
end
------------------ Love Stuff --------------------

function beginContact(a, b, coll)
   local au = a:getUserData()
   local bu = b:getUserData()

   if au then 
      au:beginCollide(b, coll)
   end
   
   if bu then
      bu:beginCollide(a, coll)
   end
end

function endContact(a, b, coll)
   local au = a:getUserData()
   local bu = b:getUserData()

   if au then 
      au:endCollide(b, coll)
   end
   
   if bu then
      bu:endCollide(a, coll)
   end
end

function love.load() 
   love.physics.setMeter(64)
   world = love.physics.newWorld(0, 35*64, true)
   world:setCallbacks(beginContact, endContact)

   ground = {}
   ground.body = love.physics.newBody(world, 800/2, 600-50/2)
   ground.shape = love.physics.newRectangleShape(800, 50)
   ground.fixture = love.physics.newFixture(ground.body, ground.shape)

   player0 = Player:new()

   player0.body = love.physics.newBody(world, 200, 150, "dynamic")
   player0.shape = love.physics.newRectangleShape(0, 0, 50, 50)
   player0.fixture = love.physics.newFixture(player0.body, player0.shape, 0)
   player0.fixture:setUserData(player0)

   love.graphics.setBackgroundColor(104, 136, 248)
   love.graphics.setMode(800, 600, false, true, 0)
end

function love.update(dt)
   world:update(dt)

   if love.keyboard.isDown("up") then
      player0:jump()
   elseif love.keyboard.isDown("down") then
      
   else
      
   end
   
   if love.keyboard.isDown("left") then
      player0:walkLeft()
   elseif love.keyboard.isDown("right") then
      player0:walkRight()
   else
      player0:stop()
   end 


end

function love.draw()
   love.graphics.setColor(50, 50, 100)
   love.graphics.polygon("fill", player0.body:getWorldPoints(player0.shape:getPoints()))

   love.graphics.setColor(72, 160, 14) 
   love.graphics.polygon("fill", ground.body:getWorldPoints(ground.shape:getPoints())) 
end